package vc.thinker.mqtt.proxyclient;

import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Component;
import vc.thinker.absctacts.mqtt.protocol.AbstractCommonEntityHandleMqttProtocol;
import vc.thinker.absctacts.mqtt.topic.TopicDefinition;
import vc.thinker.mqtt.config.ThinkerMqttConnectionProperties;
import vc.thinker.mqtt.proxyclient.enums.ThirdPartyProxyTopicDefinition;

/**
 * 第三方协议代理协议 客户端
 *
 * @author HeTongHao
 * @since 2019-10-29 16:00
 */
@Slf4j
@Component
@ConditionalOnBean(ThinkerMqttConnectionProperties.class)
public class ThirdPartyProxyClientProtocol extends AbstractCommonEntityHandleMqttProtocol {
    public ThirdPartyProxyClientProtocol(ThinkerMqttConnectionProperties thinkerMqttConnectionProperties) {
        super(thinkerMqttConnectionProperties);
    }

    @Override
    public String[] subscribe() {
        return new String[]{
                ThirdPartyProxyTopicDefinition.executeService.getTopic(),
                ThirdPartyProxyTopicDefinition.meterListPullResponse.getTopic(),
                ThirdPartyProxyTopicDefinition.meterChange.getTopic()
        };
    }

    @Override
    protected TopicDefinition resolveByTopic(String topic) {
        String protocolCode = ThirdPartyProxyTopicDefinition.resolveProtocolCodeByTopic(topic);
        String matchTopic = topic.replace(protocolCode, "+");
        return ThirdPartyProxyTopicDefinition.RESOLVE_BY_TOPIC.resolve(matchTopic);
    }

    /**
     * 统一发送Mqtt消息入口
     *
     * @param topic        topic定义
     * @param protocolCode 第三方协议编号
     * @param payload      数据荷载
     */
    public void sendMessage(ThirdPartyProxyTopicDefinition topic, String protocolCode, byte[] payload) {
        super.getMqttCommandPush().sendMqttMessage(topic.genTopic(protocolCode), payload);
    }
}
